/* g_o_l.c the functions used in the Game of Life

 * Copyright (C) 2003 -- Lesley Shannon -- University of Toronto

 * This software is available to the user without any license fee or
 * royalty on an "as is" basis with no guarantee of support.   The author 
 * and University of Toronto disclaim any and all warranties, whether 
 * express, implied or statuary, including any implied warranties or
 * merchantability or of fitness for a particular pupose.
 *
 * Only not-for-profit use of this software is permitted.  The user may
 * modify or use the source code for other not-for-profit research
 * endeavours, provided that all copyright attribution on the source code is
 * retained, and the original or modified source code is not redistributed,
 * in whole or in part, or included in or with any commerical product, except
 * by written agreement with the authors, and full and complete attribution
 * for use of the code is given in any resulting publications.
 */

/*Programmer: Lesley Shannon
 *Date: July 31, 2002
 *File: g_o_l.c
 *
 *Description:
 *
 *The for loop provides a useful control intensive kernel for testing a
 *processor.  Presently, the next state is always assigned to the present 
 *state.  There are a few things that I feel should be considered:
 *
 * *1) it may be a better implementation to use an event flag to flag the 
 * need for the present state to be over written. 
 *
 * *2) the get_neighbor_cnt should maybe reworked (more efficient)
 *
 * *3) the entire structure could be stored in one char.  However accessing
 * a field of information would become a more involved process.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include "g_o_l.h"

void adjust_neigh_cnt(char pres_state, int i, int j)
{
	if (pres_state==ALIVE) /*Increase everyone's neighbor count.*/
	{
		if (i==0)
		{
			if (j==0)
			{
				game_grid[0][1].neigh_cnt++;
				game_grid[1][1].neigh_cnt++;
				game_grid[1][0].neigh_cnt++;
			}
			else if (j==(M-1))
			{
				game_grid[0][M-2].neigh_cnt++;
				game_grid[1][M-2].neigh_cnt++;
				game_grid[1][M-1].neigh_cnt++;
			}
			else
			{
				game_grid[0][j-1].neigh_cnt++;
				game_grid[0][j+1].neigh_cnt++;
				game_grid[1][j-1].neigh_cnt++;
				game_grid[1][j].neigh_cnt++;
				game_grid[1][j+1].neigh_cnt++;
			}
		}
		else if (i==(N-1))
		{
			if (j==0)
			{
				game_grid[N-2][0].neigh_cnt++;
				game_grid[N-2][1].neigh_cnt++;
				game_grid[N-1][1].neigh_cnt++;
			}
			else if (j==(M-1))
			{
				game_grid[N-2][M-1].neigh_cnt++;
				game_grid[N-2][M-2].neigh_cnt++;
				game_grid[N-1][M-2].neigh_cnt++;
			}
			else
			{
				game_grid[N-2][j-1].neigh_cnt++;
				game_grid[N-2][j].neigh_cnt++;
				game_grid[N-2][j+1].neigh_cnt++;
				game_grid[N-1][j-1].neigh_cnt++;
				game_grid[N-1][j+1].neigh_cnt++;
			}
		}
		else if (j==0)
		{
			game_grid[i-1][0].neigh_cnt++;
			game_grid[i-1][1].neigh_cnt++;
			game_grid[i][1].neigh_cnt++;
			game_grid[i+1][1].neigh_cnt++;
			game_grid[i+1][0].neigh_cnt++;
		}
		else if (j==(M-1))
		{
			game_grid[i-1][M-1].neigh_cnt++;
			game_grid[i-1][M-2].neigh_cnt++;
			game_grid[i][M-2].neigh_cnt++;
			game_grid[i+1][M-2].neigh_cnt++;
			game_grid[i+1][M-1].neigh_cnt++;
		}
		else
		{
			game_grid[i-1][j-1].neigh_cnt++;
			game_grid[i-1][j].neigh_cnt++;
			game_grid[i-1][j+1].neigh_cnt++;
			game_grid[i][j-1].neigh_cnt++;
			game_grid[i][j+1].neigh_cnt++;
			game_grid[i+1][j-1].neigh_cnt++;
			game_grid[i+1][j].neigh_cnt++;
			game_grid[i+1][j+1].neigh_cnt++;
		}
	}

	return;
}

void init_grid(void)
{
	int i, j;	/*Counters*/

	/* Initialize the grid to a known "empty" state.*/
	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			game_grid[i][j].pres_state = DEAD;
			game_grid[i][j].next_state = DEAD;
			game_grid[i][j].neigh_cnt = 0;
			/*game_grid.event = DEAD;*/
		}
	}
	return;
}

void seed_the_grid(void)
{
	char file_char;
	int i, j;

	printf("Seeding the grid.\n");

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			file_char = getc(InFile);
			/*printf("%c", file_char);*/
			if (file_char== '0')
				game_grid[i][j].pres_state = DEAD;
			else
				game_grid[i][j].pres_state = ALIVE;
			adjust_neigh_cnt(game_grid[i][j].pres_state, i, j);
		}
		file_char = getc(InFile); /*To remove the EOL character.*/
		/*printf("%c", file_char);*/
	}
	return;
}

void set_cell_next_state(char neigh_cnt, int i, int j)
{

	if ((neigh_cnt<2) || (neigh_cnt>3))
		game_grid[i][j].next_state = DEAD;
	else if (neigh_cnt==2)
		game_grid[i][j].next_state = game_grid[i][j].pres_state;
	else /*neigh_cnt = 3*/
		game_grid[i][j].next_state = ALIVE;

	game_grid[i][j].neigh_cnt = 0;

	return;
}

void set_grid_next_state(void)
{
	int i, j;
	/* Calculate the next generation of "Life". */
	/*printf("Calculate the next state.\n");*/

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			set_cell_next_state(game_grid[i][j].neigh_cnt, i, j);
		}
	}

	return;
}

void set_new_grid_pres_state(void)
{
	int i,j;

	/*Make the next generation the present one.*/
	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			game_grid[i][j].pres_state = 
				game_grid[i][j].next_state;
		        adjust_neigh_cnt(game_grid[i][j].pres_state, i, j);
		}
	}
	return;
}

void write_outfile(void)
{
	int i, j; /*Used as counters.*/

	/*Read through the grid and print it to the screen.*/
	printf("Writing Output File.\n\n");

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			if (game_grid[i][j].pres_state==DEAD)
				fprintf(OutFile,"0");
			else
				fprintf(OutFile,"1");

		}
		fprintf(OutFile,"\n");
	}
	return;
}

/** FOR DEBUGGING ONLY **/
void display_present_generation(int generation)
{
	int i, j; /*Used as counters.*/

	/*Read through the grid and print it to the screen.*/
	printf("Display generation %d of life:\n\n", generation);

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			if (game_grid[i][j].pres_state==DEAD)
				printf("0");
			else
				printf("1");

		}
		printf("\n");
	}
	printf("\n");
	return;
}

/** FOR DEBUGGING ONLY **/
void display_next_generation(int generation)
{
	int i, j; /*Used as counters.*/

	/*Read through the grid and print it to the screen.*/
	printf("Display generation %d of life (NEXT GEN):\n\n", generation);

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			if (game_grid[i][j].next_state==DEAD)
				printf("0");
			else
				printf("1");

		}
		printf("\n");
	}
	printf("\n");
	return;
}

/** FOR DEBUGGING ONLY **/
void display_neigh_cnts(int generation)
{
	int i, j; /*Used as counters.*/
	int cnt;

	/*Read through the grid and print it to the screen.*/
	printf("Display the neighbor counts for generation %d of life:\n\n", 
			generation);

	for (i=0; i<N; i++)
	{
		for (j=0; j<M; j++)
		{
			cnt = (int)(game_grid[i][j].neigh_cnt);
			printf("%d", cnt);
		}
		printf("\n");
	}
	printf("\n");
	return;
}

