/* g_o_l.h the header file for functions used in the game of life

 * Copyright (C) 2003 -- Lesley Shannon -- University of Toronto

 * This software is available to the user without any license fee or
 * royalty on an "as is" basis with no guarantee of support.   The author 
 * and University of Toronto disclaim any and all warranties, whether 
 * express, implied or statuary, including any implied warranties or
 * merchantability or of fitness for a particular pupose.
 *
 * Only not-for-profit use of this software is permitted.  The user may
 * modify or use the source code for other not-for-profit research
 * endeavours, provided that all copyright attribution on the source code is
 * retained, and the original or modified source code is not redistributed,
 * in whole or in part, or included in or with any commerical product, except
 * by written agreement with the authors, and full and complete attribution
 * for use of the code is given in any resulting publications.
 */

/*Author: Lesley Shannon
 *Date: July 31, 2002
 *File: g_o_l.h 
 *
 *Description:
 *
 *The header file for John Conway's Game of Life.  It defines necessary 
 *constants and functions that may be used for debugging.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdio.h>


#define N 10  /*Number of rows*/
#define M 20  /*Number of columns*/

#define ALIVE 1
#define DEAD  0
#define NUM_PASSES 1000000


char *InFileName;
FILE *InFile;

char *OutFileName;
FILE *OutFile;

/*Used to represent a single cell in the the grid.*/
typedef struct Cell_type
{
	char pres_state;
	char next_state;
	char neigh_cnt;
} Cell_t;

Cell_t game_grid [N][M];
/*Cell_t **game_grid;*/

void adjust_neigh_cnt(char pres_state, int i, int j);

void init_grid(void);

void seed_the_grid(void);

void set_cell_next_state(char neigh_cnt, int i, int j);

void set_grid_next_state(void);

void set_new_grid_pres_state(void);

void write_outfile(void);

/** FOR DEBUGGING ONLY> **/
void display_present_generation(int generation);

/** FOR DEBUGGING ONLY> **/
void display_next_generation(int generation);

/** FOR DEBUGGING ONLY> **/
void display_neigh_cnts(int generation);

